require 'ety/version'
require 'nokogiri'
require 'open-uri'

class EtyError < StandardError; end

module Ety
  extend self

  def entry(args)
    if !(args & %w[--help -h]).empty? or args.empty?
      help
    end

    @word=args.pop
    etymology, related_words=fetch_site
    puts "\e[1;4mEtymology of \"#{word}\":\e[m\n#{etymology}"
    puts "\n\e[1;4mRelated words:\e[m\n#{related(related_words).join(', ')}"
  end

  def help
    puts <<~eof
    Usage: ety <word>
    eof
    exit
  end

  def fetch_site
    html=Nokogiri::HTML(open(url,headers))
    pattern='//section[starts-with(@class,"word__defination")]'
    content=html.xpath(pattern)
    etymology=content[0].css('p').text
    related_words=content.map {|s| s.css('span').map(&:text)}.flatten!
    [etymology,related_words]
  end

  def related(words)
    str=''
    words.chunk {|e| e[0]=='-'}
         .flat_map do |tf,a|
      if tf
        str+a[0][1..-1]
      else
        a.size>1 ? [str=a.pop, a].last : a
      end
    end
  end

  def word
    @word
  end

  def url
    "https://www.etymonline.com/search?q=#{word}"
  end

  def headers
    {'User-Agent' => 'ety-rubygem', 'accept' => 'text/html'}
  end
end
